/***********************************************************************************************************************
 *
 * blueBill Resources - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it
 * SCM: https://kenai.com/hg/bluebill~resources-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.taxonomy.birds.ebn.italia;

import javax.annotation.Nonnull;
import java.util.Locale;
import java.util.Scanner;
import java.io.BufferedReader;
import java.io.InputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.util.logging.Logger;
import org.openrdf.repository.Repository;
import org.openrdf.repository.RepositoryException;
import it.tidalwave.bluebill.taxonomy.Taxonomy;
import it.tidalwave.bluebill.taxonomy.Taxon;
import it.tidalwave.bluebill.taxonomy.birds.BirdLocaleInfo;
import it.tidalwave.bluebill.taxonomy.birds.BirdTaxonomyImporter;
import it.tidalwave.bluebill.taxonomy.birds.impl.PatchedDefaultMutableDisplayable;
import static it.tidalwave.bluebill.taxonomy.Taxon.Rank.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class EbnItalia2003Importer extends BirdTaxonomyImporter
  {
    private static final String CLASS = EbnItalia2003Importer.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    private static final String[] LANGUAGES = { "en", "en_us", "it", "de", "da", "no", "fi", "sv", "fr", "es", "nl" };

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    class LanguageInitializer extends PatchedDefaultMutableDisplayable
      {
        public LanguageInitializer (final @Nonnull String[] languages,
                                    final @Nonnull String italianName,
                                    final @Nonnull String englishName,
                                    final @Nonnull BirdLocaleInfo localeInfo,
                                    final @Nonnull String scientificName)
          {
            for (final String language : languages)
              {
                String localizedName = "";

                try
                  {
                    // Italian and english are taken from the EBN checklist;
                    if ("it".equals(language))
                      {
                        localizedName = italianName;
                      }
                    else if ("en".equals(language))
                      {
                        localizedName = englishName;
                      }
                    else
                      {
                        localizedName = localeInfo.getLocalizedName(scientificName, language);
                      }
                  }
                catch (NotFoundException e)
                  {
                    try
                      {
                        localizedName = localeInfo.getLocalizedName(italianName, language);
                      }
                    catch (NotFoundException e2)
                      {
                        if ("it".equals(language))
                          {
                            localizedName = italianName;
                          }
                        else if ("en".equals(language))
                          {
                            localizedName = englishName;
                          }
                        else
                          {
                            localizedName = "n.a. (" + scientificName + ")";
                          }

                        logger.fine("                 Missing translation for: " + language);
                        addMissingTranslation(scientificName + " / " + italianName, language);
                        continue;
                      }
                  }

                setDisplayName(localizedName, new Locale(language));
//                RDFUtils.setDisplayName(taxon.as(Entity.class), localizedName, new Locale(language));
              }
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public Taxonomy run (final @Nonnull Repository repository,
                         final @Nonnull String taxonomyName,
                         final @Nonnull String idPrefix)
      throws Exception
      {
        init(repository);

        final Taxonomy taxonomy = createTaxonomy(repository, idPrefix, EbnItalia2003Importer.class, LANGUAGES);
        final Taxon aves = createAves(taxonomy, idPrefix);
        
        final InputStream is = getClass().getResourceAsStream(sourceResourceName);
        final BufferedReader br = new BufferedReader(new InputStreamReader(is, "UTF-8"));
        process2003(br, taxonomy, aves, birdLocaleInfo, idPrefix);
        br.close();
        taxonUniqueIdManager.close();

        return taxonomy;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private void process2003 (final @Nonnull BufferedReader br,
                              final @Nonnull Taxonomy taxonomy,
                              final @Nonnull Taxon aves,
                              final @Nonnull BirdLocaleInfo localeInfo,
                              final @Nonnull String idPrefix)
      throws IOException, RepositoryException
      {
        initialize();
        
        br.readLine();
        br.readLine();
        br.readLine();

        Taxon order = null;
        Taxon family = null;
//        TaxonomyConcept subFamily = null;
        String orderPath = "";
        String familyPath = "";
//        String subFamilyPath = "";

        for (;;)
          {
            String s = br.readLine();

            if (s == null)
              {
                break;
              }

            s = s.trim();
            s = taxonTranslator.fixTypos(s);

            if (s.equals(""))
              {
                continue;
              }

            if (s.equals("$$$$$$$$$$") || s.equals("$$0$$$$$$$$") 
                || s.startsWith("segnando nella")
                || s.startsWith("X: specie osservata")
                || s.startsWith("H: specie solo sentita")
                || s.startsWith("#: specie osservata"))
              {
                continue;
              }

//            System.err.println(s);
            final Scanner scanner = new Scanner(s).useDelimiter("\\$");

            if (s.startsWith("-"))
              {
                scanner.next();
                final String orderName = capitalized(scanner.next().trim());
                final String familyName = capitalized(scanner.next().trim());
                logger.info("**************** ORDER & FAMILY: *%s* *%s*", orderName, familyName);

                orderPath = "/Animalia/Chordata/Aves/" + orderName;
                familyPath = orderPath + "/" + familyName;
                
                order = findOrCreateSubTaxon(aves, idPrefix, orderPath).withRank(ORDER).
                                                                        withScientificName(orderName).
                                                                        withSpecificEpythet(orderName).
                                                                        build();
                family = findOrCreateSubTaxon(order, idPrefix, familyPath).withRank(FAMILY).
                                                                           withScientificName(familyName).
                                                                           withSpecificEpythet(familyName).
                                                                           build();
//                subFamily = null;
              }
            else
              {
                final String italianName = scanner.next();                
                final String scientificName = scanner.next().replace("(", "").replace(")", ""); // FIXME
                final String englishName = scanner.next();
                
                final Scanner scanner2 = new Scanner(scientificName).useDelimiter(" ");
                final String genusName = scanner2.next();
                final String speciesName = scanner2.next();

                final String genusPath = familyPath + "/" + genusName;
                final String speciesPath = genusPath + "/" + speciesName;

                final LanguageInitializer commonNames = new LanguageInitializer(LANGUAGES, italianName, englishName, localeInfo, scientificName);
                final Taxon genus = findOrCreateSubTaxon(family, idPrefix, genusPath).withRank(GENUS).
                                                                                      withScientificName(genusName).
                                                                                      withSpecificEpythet(genusName).
                                                                                      build();
                final Taxon species = findOrCreateSubTaxon(genus, idPrefix, speciesPath).withRank(SPECIES).
                                                                                         withScientificName(genusName + " " + speciesName).
                                                                                         withSpecificEpythet(speciesName).
                                                                                         withDisplayNames(commonNames).
                                                                                         build();
              }
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    private static String capitalized (final @Nonnull String string)
      {
        return string.substring(0, 1).toUpperCase() + string.substring(1).toLowerCase();
      }
  }
